;;; ----------------------------------------------------------------------------
;;; $RCSfile: DemoASC.lisp,v $
;;; $Author: hubert $
;;; $Date: 2000/05/05 06:13:41 $
;;; $Revision: 1.1.1.1 $
;;; ----------------------------------------------------------------------------

;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)

(declaim (optimize (speed 3)
		   (compilation-speed 0)
		   (safety 0)
		   (debug 0)))



(load "./glos-library.x86f")
(load "./ascparser-library.x86f")


(defun test-asc ( &rest args &key (host "localhost")
			(string "Asc parser")
			(font "fixed")
			(file "asc/dolphin.asc")
			(zoom 1000.0))
  (let ((display nil)
	(abort t))
    (unwind-protect
	(progn 
	  (setf display (open-display host))
	  (multiple-value-prog1
	    (let* ((obj-list (load-list-asc file))
		   (dolphin (car obj-list))
		   
		   (screen (display-default-screen display))
		   (black (screen-black-pixel screen))
		   (white (screen-white-pixel screen))
		   (font (open-font display font))
		   (width *screen-width*)
		   (height *screen-height*)
		   (x (truncate (- (screen-width screen) width) 2))
		   (y (truncate (- (screen-height screen) height) 2))
		   (window (create-window :parent (screen-root screen)
					  :x x :y y :width width :height height
					  :background black
					  :border white
					  :border-width 1
					  :colormap (screen-default-colormap screen)
					  :bit-gravity :center
					  :event-mask '(:exposure :button-press)))
		   (gcontext (create-gcontext :drawable window
					      :background black
					      :foreground white
					      :font font)))
	      ;; Set window manager hints
	      (set-wm-properties window
				 :name 'Test-Begin
				 :icon-name string
				 :resource-name string
				 :resource-class 'Test-GLOS
				 :command (list* 'Test-GLOS host args)
				 :x x :y y :width width :height height
				 :min-width width :min-height height
				 :input :off :initial-state :normal)
	      (init-glos)
	      ;(ViewPort  0 0 160 100)
	      (ShadeModel :SMOOTH)

	      (Enable :LIGHTING)
	      (Enable LIGHT0)
	      (Enable LIGHT1)
	      (Enable LIGHT2)

	      (Light LIGHT0 :POSITION 0.5 0.0 0.5 0.0)
	      (Light LIGHT0 :SPECULAR 1.0 1.0 1.0 1.0)

	      (Light LIGHT1 :POSITION 0.5 -0.5 0.5 0.0)
	      (Light LIGHT1 :DIFFUSE 0.0 0.0 0.0 1.0)
	      (Light LIGHT1 :AMBIANT 0.0 0.0 0.0 1.0)
	      (Light LIGHT1 :SPECULAR 0.117 0.392 0.392 1.0)

	      (Light LIGHT2 :POSITION -0.5 -0.5 -0.5 0.0)
	      (Light LIGHT2 :DIFFUSE 0.157 0.157 0.253 1.0)
	      (Light LIGHT2 :AMBIANT 0.157 0.157 0.253 1.0)
	      (Light LIGHT2 :SPECULAR 0.0 0.0 1.0 1.0)

	      (Material :SPECULAR 1.0 1.0 1.0 1.0)
	      (Material :SHININESS 128.0)

	      (Enable :DEPTH-BUFFER)
	      ;(Disable  :NORMALIZE)
	      (ClearColor 0 0 0)
	      (MatrixMode :PROJECTION)
	      (LoadIdentity)
	      ;(Frustum -1.0 1.0 -1.0 1.0 1.5 20.0)
	      (Perspective 60.0 (coerce (/ *viewport-width* *viewport-height*) 'single-float) 100.0 2000.0)
	      (MatrixMode :MODELVIEW)
	      (map-window window) 
	      ;; Handle events
	      (event-case (display :discard-p t :force-output-p t)
			  (exposure  ;; Come here on exposure events
			   (window count)
			   (when (zerop count)
			     (dotimes (i 360)
			       (let ((angle (* i (/ 3.14159 180.0))))
				 (declare (type single-float angle))
				 
				 (Clear :COLOR-BUFFER :DEPTH-BUFFER)
				 				 
				 (LoadIdentity)
				 (LookAt 0.0 0.0 zoom
					 0.0 0.0 0.0
					 0.0 1.0 0.0)
				 ;(Translate 0.0 0.0 -1500.0)
				 
				 (Rotate angle 0.0 1.0 0.0)
				 (Rotate 1.5707963267948966 1.0 0.0 0.0)
				 (Color 157 157 253)
				 (Begin-object dolphin)

				 (put-image window gcontext *image*
					    :x 0 :y 0 :width *screen-width* :height *screen-height*)))))
				 
			     ;; Ignore all but the last exposure event
			     ;; Returning non-nil causes event-case to exit
			  (button-press () t))
	      (setf abort nil))))
      ;; Ensure display is closed when done
      (when display
	(close-display display :abort abort)))))
