;;; ----------------------------------------------------------------------------
;;; $RCSfile: Glos.lisp,v $
;;; $Author: hubert $
;;; $Date: 2000/05/05 06:13:38 $
;;; $Revision: 1.1.1.1 $
;;; ----------------------------------------------------------------------------

;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)
 
;;; Global variables that must be accessible everywhere

(declaim (optimize (speed 3)
		   (compilation-speed 0)
		   (safety 0)
		   (debug 0)))

(use-package :xlib)

(defparameter *screen-width* 320)
(defparameter *screen-height* 200)

(defparameter *screen-centerx* (truncate *screen-width* 2))
(defparameter *screen-centery* (truncate *screen-height* 2))

(defparameter *near* 10)
(defparameter *far* 2000)


;;; color and depth buffers
(defparameter *imagebuf* (make-array (list *screen-height* *screen-width*) :element-type 'card16))
(defparameter *zbuffer* (make-array (list *screen-height* *screen-width*) :element-type 'int32))
(defparameter *image* (create-image :data *imagebuf* :depth 16))

(declaim (type (simple-array card16) *imagebuf*)
	 (type (simple-array int32) *zbuffer*)
	 (type card32 *near* *far*)
	 (type card16 *screen-width* *screen-height* *screen-centerx* *screen-centery*))


;;; lighting variables

(defparameter *nblight* 8)
(declaim (type card8 *nblight*))


;;; state variables

(defparameter *clear-color* 0)

(defparameter *do-lighting* nil)
(defparameter *use-zbuffer* t)
(defparameter *do-normals-normalize* nil)

(defparameter *shade-model* :FLAT)

(declaim (type card16 *clear-color*))
(declaim (type symbol *shade-model*))

(defconstant +state-keys+ '(:LIGHTING :DEPTH-BUFFER :NORMALIZE))
(defconstant +state-vars+ '(*do-lighting* *use-zbuffer* *do-normals-normalize*))


;;; state handling functions

(defmacro make-toggle (macroname bool)
  `(defmacro ,macroname (mode)
     (if (and (numberp (eval mode))
	      (>= (eval mode) 0)
	      (< (eval mode) *nblight*))
	 `(setf (aref *tlightsbool* ,mode) ,,bool)
       (if (not (member mode +state-keys+))
	   (error "~S: unknown mode: ~S" (string ',macroname) mode)
	 `(setf ,(elt +state-vars+ (position mode +state-keys+)) ,,bool)))))
	     
(make-toggle Enable t)
(make-toggle Disable nil)


(defmacro ShadeModel (model)
  "Set the shade model"
  (if (or (eq model :FLAT) (eq model :SMOOTH))
      `(setf *shade-model* ,model)
    (error "ShadeModel: unknown argument")))



;;; viewport variables

(defvar *viewport-width*)
(defvar *viewport-height*)
(defvar *viewport-half-width*)
(defvar *viewport-half-height*)
(defvar *viewport-centerx*)
(defvar *viewport-centery*)
(declaim (type card16 *viewport-width* *viewport-height* *viewport-half-width* *viewport-half-height*
	       *viewport-centerx* *viewport-centery*))

(defun ViewPort (x y w h)
  "Set the viewport to be a rectangle of width w and height h (in pixels). its
  upper-left corner is at (x,y) in pixels"
  (declare (type card16 x y w h))
  (setf *viewport-width*   w
	*viewport-height*  h
	*viewport-half-width*  (truncate w 2)
	*viewport-half-height* (truncate h 2))
  (setf *viewport-centerx* (+ x *viewport-half-width*)
	*viewport-centery* (+ y *viewport-half-height*)))
  
(defun init-glos ()
  "Initialize the glos library to default state"
  (Disable :LIGHTING)
  (Disable :NORMALIZE)
  (Disable :DEPTH-BUFFER)
  (ShadeModel :FLAT)
  (ViewPort 0 0 *screen-width* *screen-height*))


