;;; ----------------------------------------------------------------------------
;;; $RCSfile: Lighting.lisp,v $
;;; $Author: hubert $
;;; $Date: 2000/05/05 06:13:41 $
;;; $Revision: 1.1.1.1 $
;;; ----------------------------------------------------------------------------

;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)

(declaim (optimize (speed 3)
		   (compilation-speed 0)
		   (safety 0)
		   (debug 0)))


(defstruct material
  (ambiant   (vec4-make 0.2 0.2 0.2 1.0) :type vec4)
  (diffuse   (vec4-make 0.8 0.8 0.8 1.0) :type vec4)
  (specular  (vec4-make 0.0 0.0 0.0 1.0) :type vec4)
  (shininess 0.0                         :type single-float)
  (emission  (vec4-make 0.0 0.0 0.0 1.0) :type vec4))


(defstruct light
  (ambiant        (vec4-make 0.0 0.0 0.0 1.0) :type vec4)
  (diffuse        (vec4-make 1.0 1.0 1.0 1.0) :type vec4)
  (specular       (vec4-make 1.0 1.0 1.0 1.0) :type vec4)
  (position       (vec4-make 0.0 0.0 1.0 0.0) :type vec4)
  (spot_direction (vec3-make 0.0 0.0 -1.0)    :type vec3)
  (spot_exponent  0.0        :type single-float)
  (sport_cutoff   180.0      :type single-float)
  (constant_attenuation  1.0 :type single-float)
  (linear_attenuation    0.0 :type single-float)
  (quadratic_attenuation 0.0 :type single-float))


(defparameter *current-material* (make-material))
(defparameter *tlights* (make-array *nblight* :element-type 'light))
(defparameter *tlightsbool* (make-array *nblight* :element-type 'boolean))
(declaim (type material *current-material*)
	 (type (simple-vector *) *tlights*)
	 (type (simple-vector *) *tlightsbool*))


;; allocates memory for the tab light to avoid consing
(dotimes (i *nblight*)
  (declare (type card8 i))
  (setf (aref *tlights* i) (make-light)
	(aref *tlightsbool* i) nil))


(defconstant LIGHT0 0)
(defconstant LIGHT1 1)
(defconstant LIGHT2 2)
(defconstant LIGHT3 3)
(defconstant LIGHT4 4)
(defconstant LIGHT5 5)
(defconstant LIGHT6 6)
(defconstant LIGHT7 7)


(defmacro Light (light-num slot-type &rest slot-values)
  (let ((slot (symb "LIGHT-" slot-type))
	(first-val (first slot-values))
	(second-val (second slot-values))
	(third-val (third slot-values))
	(fourth-val (fourth slot-values)))
    (case slot-type
      ((:AMBIANT :DIFFUSE :SPECULAR :POSITION)
       `(setf (vec-x (,slot (aref *tlights* ,light-num))) ,first-val
	      (vec-y (,slot (aref *tlights* ,light-num))) ,second-val
	      (vec-z (,slot (aref *tlights* ,light-num))) ,third-val
	      (vec-w (,slot (aref *tlights* ,light-num))) ,fourth-val))
      (:SPOT_DIRECTION
       `(setf (vec-x (,slot (aref *tlights* ,light-num))) ,first-val
	      (vec-y (,slot (aref *tlights* ,light-num))) ,second-val
	      (vec-z (,slot (aref *tlights* ,light-num))) ,third-val))
      (t
       `(setf (,slot (aref *tlights* ,light-num)) ,first-val)))))


(defmacro Material (slot-type &rest slot-values)
  (let ((slot (symb "MATERIAL-" slot-type))
	(first-val (first slot-values))
	(second-val (second slot-values))
	(third-val (third slot-values))
	(fourth-val (fourth slot-values)))
    (case slot-type
      (:SHININESS
       `(setf (,slot *current-material*) ,first-val))
      (t
       `(setf (vec-x (,slot *current-material*)) ,first-val
	      (vec-y (,slot *current-material*)) ,second-val
	      (vec-z (,slot *current-material*)) ,third-val
	      (vec-w (,slot *current-material*)) ,fourth-val)))))
       




