;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)

(in-package :cl-user)

;;; given a system definition, create a single file by concatenating all
;;; the object files that comprise that system - in the correct order

(defun copy-stream (from to)
  "Copy into TO from FROM until end of file, without translating or otherwise mauling anything"
  (let ((buf (make-array 4096 :element-type (stream-element-type from)
                         :initial-element #\Space)))
    (do ((pos (read-sequence buf from)  (read-sequence buf from)))
        ((= 0 pos) nil)
      (write-sequence buf to :end pos))))

(defun print-binary-file-operation (component force)
  "Spit the binary file associated with COMPONENT to *STANDARD-OUTPUT*"
  (with-open-file (i (compile-file-pathname (make::component-pathname component :binary))
                     :direction :input)
    (copy-stream i *standard-output*))
  nil)

(defun agglomerate-system (system output-file)
  "List files in SYSTEM, in the order that they need to be processed"
  (make::component-operation 'print-binary 'print-binary-file-operation)
  (with-open-file (o output-file :direction :output
                     :if-exists :rename)
    (let ((*standard-output* o))
      (make::operate-on-system  system 'print-binary))))


(defmacro compile-glos (&rest args)
  `(progn (compile-system 'glos ,@args)
	  (agglomerate-system 'glos "glos-library.x86f")))

  
(defmacro compile-ascparser (&rest args)
  `(progn (compile-system 'ascparser ,@args)
	  (agglomerate-system 'ascparser "ascparser-library.x86f")))





