;;; ----------------------------------------------------------------------------
;;; $RCSfile: IOStream.lisp,v $
;;; $Author: hubert $
;;; $Date: 2000/05/05 06:13:58 $
;;; $Revision: 1.1.1.1 $
;;; ----------------------------------------------------------------------------

;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)

(declaim (optimize (speed 0)
		   (compilation-speed 0)
		   (safety 0)
		   (debug 3)))


;;;-----------------------------------------------------------------------------
;;; Tool functions from Paul Graham
;;;-----------------------------------------------------------------------------


;;Paul Graham's 'Ansi Common Lisp'
(defun tokens (str test start)
  (let ((p1 (position-if test str :start start)))
    (if p1
	(let ((p2 (position-if #'(lambda (c) 
				   (not (funcall test c)))
			       str :start p1)))
	  (cons (subseq str p1 p2)
		(if p2 
		    (tokens str test p2) 
		  nil)))
      nil)))


;;;-----------------------------------------------------------------------------
;;; String Functions
;;;-----------------------------------------------------------------------------


(defun str-exclude (x)
  "return nil if the argument is a character which should not appear in the
final string."
  (or (eq #\SPACE x)
      (eq #\" x)
      (eq #\, x)))


(defun str-tokenize (string)
  "return a list of tokens, ie #\SPACE-separated elements, from given string.
   Notice that encountered \"strings\" are included without double-quotes."
  (let ((nospc (lambda (x) (not (str-exclude x)))))
    (tokens string nospc 0)))


(defun str-first-token (string)
  "return the first encountered token in the given string"
  (string-right-trim
   '(#\SPACE)
   (block run
     (do* ((i 0 (1+ i))
	   (c nil)
	   (start nil)
	   (len (length string))
	   (result (make-array len :element-type 'character
		 	       :initial-element #\ ))
	   (result-pos 0)) 
	 ((>= i len) result)
       (setf c (char string i))
       (cond ((str-exclude c) (if start (return-from run result)))
	     (t (setf start t)
		(setf (char result result-pos) c)
		(incf result-pos)))))))


(defun str-fetch-arg (com string)
  "return from the given string the token just after the given one"
  (let ((com-len (length com))
	(com-pos (search com string)))
    (cond (com-pos (str-first-token (subseq string (+ com-len com-pos))))
	  (t nil))))
      
      
(defmacro str-append-to (str app)
  "appends the app string to the given string (side-effect)" 
  `(setf ,str (concatenate 'string ,str ,app)))


(defun str-append (str app)
  "returns the concatenation of given strings"
  (concatenate 'string str app))


(defun str-coerce-sf (str)
  "returns a single-float from given string if possible"
  (coerce (read-from-string str)
	  'single-float))


(defun str-coerce-r (str)
  "returns a real from given string if possible"
  (coerce (read-from-string str)
	  'real))


;;;-----------------------------------------------------------------------------
;;; Stream seeking Functions
;;;-----------------------------------------------------------------------------


(defun stream-seek (pattern stream current-line) 
  "Seeks in stream a given string pattern, if not present in the current-line string.
Returns the line of the stream where the pattern first occurs, NIL if end-of-line reached. 
Side-effet on the given stream."
  (do ((line current-line (read-line stream nil 'eof)))
      ((eq line 'eof) nil)
    (if (search pattern line) (return line))))


(defun stream-multiline-seek (start-pattern stop-pattern-list stream current-line)
  "Extension to seek-in-stream: seeks between a start pattern and any stop-pattern,
commencing from the given current-line. Returns the new current-line as a first value,
the sought string as the second value."
  (do* ((line (stream-seek start-pattern stream current-line)
	      (read-line stream nil 'eof))
	(acc nil)
	(str-seek (lambda (pat) (search pat line))))
      ((eq line 'eof) (values nil acc))
    (if (some #'identity (mapcar str-seek stop-pattern-list))
	(return (values line acc)))
    (str-append-to acc line)))

