;;; ----------------------------------------------------------------------------
;;; $RCSfile: Object.lisp,v $
;;; $Author: hubert $
;;; $Date: 2000/05/05 06:13:58 $
;;; $Revision: 1.1.1.1 $
;;; ----------------------------------------------------------------------------

;;GLOS (Graphic Library in Open Source), an ANSI Common Lisp OpenGL subset.
;;Copyright (C) 2000 the GLOS development team (http://glos.sourceforge.net)

(declaim (optimize (speed 0)
		   (compilation-speed 0)
		   (safety 0)
		   (debug 3)))


;;;-----------------------------------------------------------------------------
;;; Type Declarations
;;;-----------------------------------------------------------------------------


(deftype vertex-array ()
  `(simple-array vec4 (*)))

(deftype face-array ()
  `(simple-array glos-face (*)))


;;;-----------------------------------------------------------------------------
;;; Vertex Functions
;;;-----------------------------------------------------------------------------


(defun vertex-array-make (length)
  "build an array of null vertices"
  (make-array length
	      :element-type 'vec4
	      :initial-element nil))
  

;;;-----------------------------------------------------------------------------
;;; Object type
;;;-----------------------------------------------------------------------------


(defstruct (glos-object
	    (:constructor obj-make) 
	    (:conc-name obj-)
	    (:print-function obj-display))
  (name "Unnamed object" :type simple-base-string)
  (vertices #() :type vertex-array)
  (vertex-number 0 :type integer)
  (faces #() :type face-array)
  (fnormals #() :type (simple-array vec3 (*)))
  (vnormals #() :type (simple-array vec3 (*)))
  (face-number 0 :type integer))


(defun obj-display (o stream depth)
  (declare (ignore depth))
  (format stream "~%GLOS-Object: ~A ~%~A Vertices: ~%~A ~%~A Faces: ~%~A "
	  (obj-name o)
	  (obj-vertex-number o)
	  (obj-vertices o)
	  (obj-face-number o)
	  (obj-faces o)))


(defun obj-array-make (length)
  "build an array of null objects"
  (make-array length
	      :element-type 'glos-object
	      :initial-element nil))
   

;; calcule la normale a une face ...
(defun face-normal (a b c)
  "return the vec4 normal of the face defined by a b c"
  (let ((t1 (vec3-make (- #{a 0} #{b 0})
		       (- #{a 1} #{b 1})
		       (- #{a 2} #{b 2})))
	(t2 (vec3-make (- #{c 0} #{b 0})
		       (- #{c 1} #{b 1})
		       (- #{c 2} #{b 2})))
	(res (vec3-make 0.0 0.0 0.0)))
    (declare (type vec3 t1 t2 res))
    (vec-set-cross-product res t1 t2)
    (vec-set-normalize res)
    res))

(defun compute-normals (obj)
  "compute object normals"
  (setf (obj-fnormals obj) (make-array (obj-face-number obj) :element-type 'vec3 :initial-element (vec3-make-null)))
  (setf (obj-vnormals obj) (make-array (obj-vertex-number obj) :element-type 'vec3 :initial-element (vec3-make-null)))
  ;; calcul des normales a chaque face
  (dotimes (i (obj-face-number obj))
    (multiple-value-bind (a b c) (face-to-vertices obj i)
      (setf (aref (obj-fnormals obj) i) (face-normal c b a))))
  ;; calcul des normales a chaque vertex
  (dotimes (i (obj-face-number obj))
    (let* ((face (aref (obj-faces obj) i))
	   (ia (fac-a face))
	   (ib (fac-b face))
	   (ic (fac-c face))
	   (ismooth (fac-smoothing face))
	   (veca-sum (vec3-make-null))
	   (vecb-sum (vec3-make-null))
	   (vecc-sum (vec3-make-null))
	   (nb-veca-sum 0)
	   (nb-vecb-sum 0)
	   (nb-vecc-sum 0))
      (dotimes (j (obj-face-number obj))
	(let* ((face2 (aref (obj-faces obj) j))
	       (ja (fac-a face2))
	       (jb (fac-b face2))
	       (jc (fac-c face2))
	       (jsmooth (fac-smoothing face2)))

	  (when (= ismooth jsmooth)
	    (when (or (= ia ja)
		      (= ia jb)
		      (= ia jc))
	      (incf nb-veca-sum)
	      (incf #{veca-sum 0}
		    #{(aref (obj-fnormals obj) j) 0})
	      (incf #{veca-sum 1}
		    #{(aref (obj-fnormals obj) j) 1})
	      (incf #{veca-sum 2}
		    #{(aref (obj-fnormals obj) j) 2}))
	    
	    (when (or (= ib ja)
		      (= ib jb)
		      (= ib jc))
	      (incf nb-vecb-sum)
	      (incf #{vecb-sum 0}
		    #{(aref (obj-fnormals obj) j) 0})
	      (incf #{vecb-sum 1}
		    #{(aref (obj-fnormals obj) j) 1})
	      (incf #{vecb-sum 2}
		    #{(aref (obj-fnormals obj) j) 2}))
	    
	    (when (or (= ic ja)
		      (= ic jb)
		      (= ic jc))
	      (incf nb-vecc-sum)
	      (incf #{vecc-sum 0}
		    #{(aref (obj-fnormals obj) j) 0})
	      (incf #{vecc-sum 1}
		    #{(aref (obj-fnormals obj) j) 1})
	      (incf #{vecc-sum 2}
		    #{(aref (obj-fnormals obj) j) 2})))))
      (setf #{veca-sum 0} (/ #{veca-sum 0} nb-veca-sum))
      (setf #{veca-sum 1} (/ #{veca-sum 1} nb-veca-sum))
      (setf #{veca-sum 2} (/ #{veca-sum 2} nb-veca-sum))
      
      (setf #{vecb-sum 0} (/ #{vecb-sum 0} nb-vecb-sum))
      (setf #{vecb-sum 1} (/ #{vecb-sum 1} nb-vecb-sum))
      (setf #{vecb-sum 2} (/ #{vecb-sum 2} nb-vecb-sum))

      (setf #{vecc-sum 0} (/ #{vecc-sum 0} nb-vecc-sum))
      (setf #{vecc-sum 1} (/ #{vecc-sum 1} nb-vecc-sum))
      (setf #{vecc-sum 2} (/ #{vecc-sum 2} nb-vecc-sum))

      (vec-set-normalize veca-sum)
      (vec-set-normalize vecb-sum)
      (vec-set-normalize vecc-sum)

      (setf (aref (obj-vnormals obj) ia) veca-sum)
      (setf (aref (obj-vnormals obj) ib) vecb-sum)
      (setf (aref (obj-vnormals obj) ic) vecc-sum))))


;(defun object-to-vertices (obj)
;  "return the list of vertices of the faces of given object"
;  (let ((vxlist '())
;	(faces (obj-face-number obj)))
;    (compute-normals obj)
;    (dotimes (i faces (nreverse vxlist))
;      (multiple-value-bind (a b c) (face-to-vertices obj i)
;	(let* ((face (aref (obj-faces obj) i))
;	       (ia (fac-a face))
;	       (ib (fac-b face))
;	       (ic (fac-c face))
;	       ;(vf (aref (obj-fnormals obj) i))
;	       (va (aref (obj-vnormals obj) ia))
;	       (vb (aref (obj-vnormals obj) ib))
;	       (vc (aref (obj-vnormals obj) ic)))
;	  ;;(push `(Normal ,#{vf 0} ,#{vf 1} ,#{vf 2}) vxlist)
;	  (push `(Normal ,#{va 0} ,#{va 1} ,#{va 2}) vxlist)
;	  (push `(Vertex ,#{a 0} ,#{a 1} ,#{a 2} ,#{a 3}) vxlist)
;	  (push `(Normal ,#{vb 0} ,#{vb 1} ,#{vb 2}) vxlist)
;	  (push `(Vertex ,#{b 0} ,#{b 1} ,#{b 2} ,#{b 3}) vxlist)
;	  (push `(Normal ,#{vc 0} ,#{vc 1} ,#{vc 2}) vxlist)
;	  (push `(Vertex ,#{c 0} ,#{c 1} ,#{c 2} ,#{c 3}) vxlist))))))

;(defun object-to-vertices (obj)
;  "return the list of vertices of the faces of given object"
;  (let ((vxlist '())
;	(faces (obj-face-number obj)))
;    (dotimes (i faces (nreverse vxlist))
;      (multiple-value-bind (a b c) (face-to-vertices obj i)
;	(push a vxlist)
;	(push b vxlist)
;	(push c vxlist)))))


;;;-----------------------------------------------------------------------------
;;; Face type
;;;-----------------------------------------------------------------------------


(defstruct (glos-face
	    (:constructor fac-make)
	    (:conc-name fac-)
	    (:print-function fac-display))
  (A 0 :type fixnum)
  (B 0 :type fixnum)
  (C 0 :type fixnum)
  (AB 0 :type fixnum)
  (BC 0 :type fixnum)
  (CA 0 :type fixnum)
  (smoothing 0 :type fixnum))


(defun fac-display (f stream depth)
  (declare (ignore depth))
  (format stream "~%GLOS-Face: ~%A:~A B:~A C:~A ~%AB:~A BC:~A CA:~A ~%Smoothing ~A" 
	  (fac-a f) (fac-b f) (fac-c f)
	  (fac-ab f) (fac-bc f) (fac-ca f)
	  (fac-smoothing f)))


(defun face-array-make (length)
  "build an array of null faces"
  (make-array length
	      :element-type 'glos-face
	      :initial-element nil))


(defun face-to-vertices (obj face-num)
  "translate the n-th face of the given object into vertices"
  (let* ((face (aref (obj-faces obj) face-num))
	 (a (fac-a face))
	 (b (fac-b face))
	 (c (fac-c face))
	 (vx1 (aref (obj-vertices obj) a))
	 (vx2 (aref (obj-vertices obj) b))
	 (vx3 (aref (obj-vertices obj) c)))
    (values vx1 vx2 vx3)))
